# Generated by Django 4.2.6 on 2023-11-02 20:20

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('rps_milea', '0003_alter_mileachoices_options_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mileachoices',
            name='model',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype'),
        ),
    ]



from django.apps import apps
from django.template import Library
from django.utils.text import slugify

register = Library()

@register.simple_tag
def app_additional_config(app):
    """
    Get additional infos from the app config and build a second level menu
    """

    menu_icon = "ti ti-package"  # Default Icon
    menu_firstlevel = list()
    menu_secondlevel = list()
    app_config = apps.get_app_config(app['app_label'])

    # Menu icon
    if hasattr(app_config, 'menu_icon'):
        menu_icon = getattr(app_config, 'menu_icon')

    # Default Menu
    if not hasattr(app_config, 'menu_firstlvl') and not hasattr(app_config, 'menu_secondlvl'):
        menu_firstlevel = app['models']

    # First Level Menu
    if hasattr(app_config, 'menu_firstlvl'):
        for menu in getattr(app_config, 'menu_firstlvl'):
            for item in app['models']:
                if item['object_name'] in menu:
                    menu_firstlevel.append(item)

    # Second Lvl Menu
    if hasattr(app_config, 'menu_secondlvl'):
        for menu in getattr(app_config, 'menu_secondlvl'):
            menu_tmp = dict(name=menu[0], key=slugify(menu[0]), models=[])
            for item in app['models']:
                if item['object_name'] in menu[1]:
                    menu_tmp['models'].append(item)
            if len(menu_tmp['models']) > 0:
                menu_secondlevel.append(menu_tmp)

    return dict(
        menu_icon=menu_icon,
        menu_firstlevel=menu_firstlevel,
        menu_secondlevel=menu_secondlevel,
    )



from django import forms
from django.contrib import admin
from django.forms.widgets import Select
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _

from rps_milea.admin.defaults import MileaAdmin
from rps_milea.models.choices import MileaChoices


class TablerColorPickerWidget(Select):
    template_name = 'django/forms/widgets/colorpicker.html'


class MileaChoicesForm(forms.ModelForm):
    class Meta:
        model = MileaChoices
        fields = ('display', 'model', 'color')

    color = forms.ChoiceField(
        widget=TablerColorPickerWidget(),
        choices=MileaChoices.COLOR_CHOICES
    )


@admin.register(MileaChoices)
class MileaChoicesAdmin(MileaAdmin):
    form = MileaChoicesForm
    list_display = ('display', 'model', 'badge')
    list_display_links = ('display',)
    show_sysdata = True

    fieldsets = (
        (None, {
            'fields': (
                ('model', 'display'), 'color',
            ),
        }),
    )

    def badge(self, obj):
        return format_html(
            '<span class="badge bg-{} text-{}-fg">{}</span>', obj.color, obj.color, obj.get_color_display(),
        )
    badge.short_description = _("Color")

    def get_readonly_fields(self, request, obj=None):
        if obj:  # editing an existing object
            return self.readonly_fields + ['model',]
        return self.readonly_fields



from django.template import Library
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.contrib.admin.views.main import PAGE_VAR

register = Library()

@register.simple_tag
def milea_paginator_number(cl, i):
    """
    Generate an individual page index link in a paginated list.
    """
    if i == cl.paginator.ELLIPSIS:
        return format_html("{} ", cl.paginator.ELLIPSIS)
    else:
        return format_html(
            '<li class="page-item {}"><a class="page-link" href="{}"{}>{}</a></li>',
            'active' if i == cl.page_num else '',
            cl.get_query_string({PAGE_VAR: i}),
            mark_safe(' class="end"' if i == cl.paginator.num_pages else ""),
            i,
        )

@register.simple_tag
def setvar(val=None):
    return val



from django.apps import apps
from django.contrib import admin
from django.utils.translation import gettext_lazy as _

# Check witch auth-user app is installed:
if apps.is_installed("email_users"):
    from email_users.admin import UserAdmin
    from email_users.models import User

elif apps.is_installed("rps_milea_users"):
    from rps_milea_users.admin import UserAdmin
    from rps_milea_users.models import User


# Milea User Admin
admin.site.unregister(User)
@admin.register(User)
class NewUserAdmin(UserAdmin):
    list_display = ('email', 'first_name', 'last_name', 'last_login', 'is_active')
    list_filter = ("is_active",)  # Attention, do not allow groups, because other users will see all existing groups. Issue #10
    search_fields = ('email',)
    readonly_fields = ['last_login', 'date_joined']
    ordering = ('id',)

    fieldsets = (
        (_("Personal info"), {'fields': (('email', 'password'), ('first_name', 'last_name'),)}),
        (_("Permissions"), {"fields": ('is_active', 'is_staff', 'groups',)}),
        (_("Important dates"), {'fields': (('last_login', 'date_joined'),)}),
    )

    add_fieldsets = (
        (_("Personal info"), {'fields': ('email', ('password1', 'password2'), ('first_name', 'last_name'),)}),
        (_("Permissions"), {"fields": ('is_active', 'is_staff', 'groups',)}),
    )

    admin_fieldsets = (
        (_("Administration"), {"fields": ('is_superuser', 'user_permissions')}),
    )

    def get_fieldsets(self, request, obj=None):
        # Hole die standardmäßigen Felder, die von Django automatisch generiert werden
        fieldsets = super().get_fieldsets(request, obj=obj)

        # Füge Adminfieldset hinzu
        if request.user.is_superuser:
            fieldsets += self.admin_fieldsets

        return fieldsets

    # Modify Form
    def get_form(self, request, obj=None, **kwargs):
        form = super().get_form(request, obj, **kwargs)
        form.base_fields['is_staff'].initial = True
        form.base_fields['groups'].required = True
        return form

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        if db_field.name == 'groups' and not request.user.is_superuser:
            kwargs["queryset"] = request.user.groups.all()
            return db_field.formfield(**kwargs)
        return super().formfield_for_manytomany(db_field, request, **kwargs)

    # Modify QS for non superusers
    def get_queryset(self, request):
        qs = super().get_queryset(request)

        # Only show Objects for user group
        if not request.user.is_superuser:
            qs = qs.filter(groups__in=request.user.groups.all())
            qs = qs.filter(is_superuser=False)
            qs = qs.distinct()  # Remove multiple users if user has more than one group

        return qs



# flake8: noqa
from django.conf import settings

from .defaults import MileaAdmin
from .users import NewUserAdmin

if hasattr(settings, 'USE_MILEA_CHOICES') and settings.USE_MILEA_CHOICES:
    from .choices import MileaChoicesAdmin



from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.html import format_html
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _

from rps_milea.models.defaults import MileaModel


class MileaChoicesField(models.ForeignKey):
    """
    Define own Class so you can modify the queryset in admin.py
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class ModelSelectField(models.ForeignKey):

    def __init__(self, *args, **kwargs):

        self.allowed_apps = settings.CUSTOM_APPS

        kwargs['to'] = 'contenttypes.ContentType'
        kwargs['on_delete'] = models.CASCADE

        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        qs = ContentType.objects.filter(app_label__in=self.allowed_apps)
        defaults = {'queryset': qs}
        defaults.update(kwargs)
        return super().formfield(**defaults)


class MileaChoices(MileaModel):

    COLOR_CHOICES = [
        ('blue', 'Blue'),
        ('azure', 'Azure'),
        ('indigo', 'Indigo'),
        ('purple', 'Purple'),
        ('pink', 'Pink'),
        ('red', 'Red'),
        ('orange', 'Orange'),
        ('yellow', 'Yellow'),
        ('lime', 'Lime'),
        ('green', 'Green'),
        ('teal', 'Teal'),
        ('cyan', 'Cyan'),
        ('dark', 'Black'),
        ('muted', 'Gray')
    ]

    model = ModelSelectField()
    display = models.CharField(max_length=64, verbose_name=_("Display name"))
    color = models.CharField(max_length=16, choices=COLOR_CHOICES, verbose_name=_("Color"))
    value = models.SlugField(editable=False)

    def save(self, *args, **kwargs):
        self.value = slugify(self.display)
        super().save(*args, **kwargs)

    def __html__(self):
        return format_html('<span class="badge bg-{} text-{}-fg me-1">{}</span>', self.color, self.color, self.display)

    def __str__(self):
        return self.display

    class Meta:
        ordering = ['model', 'value']
        verbose_name = _("Selection box")
        verbose_name_plural = _("Selection boxes")



# Generated by Django 4.1.5 on 2023-10-09 07:54

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='MileaChoices',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField(db_index=True, default=True, verbose_name='Active')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('category', models.CharField(max_length=64, verbose_name='Category')),
                ('display', models.CharField(max_length=64, verbose_name='Display name')),
                ('color', models.CharField(choices=[('blue', 'Blue'), ('azure', 'Azure'), ('indigo', 'Indigo'), ('purple', 'Purple'), ('pink', 'Pink'), ('red', 'Red'), ('orange', 'Orange'), ('yellow', 'Yellow'), ('lime', 'Lime'), ('green', 'Green'), ('teal', 'Teal'), ('cyan', 'Cyan'), ('dark', 'Black'), ('muted', 'Gray')], max_length=16, verbose_name='Color')),
                ('value', models.SlugField(editable=False)),
                ('created_by', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='%(app_label)s_%(class)s_created_by', to=settings.AUTH_USER_MODEL, verbose_name='Created by')),
                ('updated_by', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='%(app_label)s_%(class)s_updated_by', to=settings.AUTH_USER_MODEL, verbose_name='Updated by')),
            ],
            options={
                'verbose_name': 'Selection',
                'verbose_name_plural': 'Selection boxes',
                'ordering': ['category', 'value'],
            },
        ),
    ]



import requests
from django import template
from django.conf import settings as core_settings
from django.contrib.staticfiles import finders

from rps_milea import settings as milea_settings

register = template.Library()

@register.simple_tag
def get_random_quote():
    """
    Funktion, die das zufällige Zitat von zenquotes.io abruft

    :return: dict with quote of the day
    """
    response = requests.get("https://zenquotes.io/api/random")
    if response.status_code == 200:
        data = response.json()
        return data[0]  # return dict with q (quote) and a (author)
    else:
        return "Willkommen"


@register.simple_tag
def custom_static(path):
    """
    Prüft ob der übergebene Dateiname in static/custom existiert.
    Falls es nicht existiert, wird die Standard Milea Datei ausgeliefert.

    :param path: path of static file
    :return: path of static file
    """
    milea_path = 'milea/' + path

    if finders.find(path):
        return core_settings.STATIC_URL + path
    else:
        return core_settings.STATIC_URL + milea_path

@register.simple_tag
def milea_setting(name: str):
    """
    Prüft ob die übergebene variable in den core settings existiert
    und gibt den value aus den settings zurück.
    Falls es nicht existiert, wird der value aus den Milea Settings ausgegeben.

    :param name: name of setting variable
    :return: value of setting variable
    """
    val = getattr(core_settings, name, None)
    if val is not None:
        return val
    return getattr(milea_settings, name, "")



# Generated by Django 4.2.6 on 2023-11-02 19:20

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('rps_milea', '0002_alter_mileachoices_created_by_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='mileachoices',
            options={'ordering': ['model', 'value'], 'verbose_name': 'Selection box', 'verbose_name_plural': 'Selection boxes'},
        ),
        migrations.RenameField(
            model_name='mileachoices',
            old_name='category',
            new_name='model',
        ),
    ]



import os
import uuid

from django.contrib.admin.models import ADDITION, CHANGE, LogEntry
from django.contrib.admin.utils import construct_change_message
from django.contrib.contenttypes.models import ContentType


def get_private_file_path(instance, filename):

    filename, file_extension = os.path.splitext(filename)
    path = str(instance._meta).replace('.', '/')
    filename = '%s%s' % (str(uuid.uuid4()), file_extension)
    return os.path.join(path, filename)


def create_log_entry(user_id, obj, new, form=None, msg=None):
    """Creates a Log Entry in the Django Admin History"""

    change_message = msg if msg is not None else construct_change_message(form, None, True if new else False)
    if user_id is None:
        user_id = 1  # Default System User

    LogEntry.objects.log_action(
        user_id,
        ContentType.objects.get_for_model(obj.__class__).id,
        obj.id,
        str(obj),
        ADDITION if new else CHANGE,
        change_message
    )



#!/usr/bin/env python

import sys

from setuptools import setup

with open('README.rst', 'r') as readme:
    long_description = readme.read()

setup(
    name='rps-milea-framework',
    version='0.0.15',
    author='red-pepper-services',
    author_email='pypi@schiegg.at',
    url='https://github.com/red-pepper-services/rps-milea-framework',
    license='MIT',
    description="Django rps.Milea Framework",
    long_description=long_description,
    include_package_data=True,
    install_requires=[
        'Django>=4.1',
        'django-admin-sortable2>=2.1.4',
        'django-object-actions>=4.1.0',
        'django-currentuser>=0.6.1',
    ],
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
)
#


# Generated by Django 4.2.6 on 2023-11-16 14:25

from django.conf import settings
from django.db import migrations
import django.db.models.deletion
import django_currentuser.db.models.fields
import django_currentuser.middleware
import rps_milea.models.choices


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('rps_milea', '0004_alter_mileachoices_model'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mileachoices',
            name='created_by',
            field=django_currentuser.db.models.fields.CurrentUserField(blank=True, default=django_currentuser.middleware.get_current_authenticated_user, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='%(app_label)s_%(class)s_created_by', to=settings.AUTH_USER_MODEL, verbose_name='Created by'),
        ),
        migrations.AlterField(
            model_name='mileachoices',
            name='model',
            field=rps_milea.models.choices.ModelSelectField(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype'),
        ),
    ]



from django.contrib import admin
from django.db import models
from django.forms import Textarea
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _
from django.contrib.contenttypes.models import ContentType

from rps_milea import settings as milea
from rps_milea.models import MileaChoices

# Default Naming
admin.site.site_header = admin.site.site_title = '%s Verwaltung' % getattr(milea, 'FW_NAME', "Django")
admin.site.index_title = 'Dashboard'


# Default Admin
class MileaAdmin(admin.ModelAdmin):

    show_sysdata = True  # Zeigt die Systemfelder an (created, updated, ...)

    list_display = ('verbose_id',)
    list_display_links = ('verbose_id',)
    readonly_fields = ['created_at', '_created_by', 'updated_at', '_updated_by',]
    search_fields = ['id',]
    list_per_page = 10
    admin_fieldsets = ()

    formfield_overrides = {
        models.TextField: {'widget': Textarea(attrs={'rows': 9})},
    }

    def _created_by(self, obj=None):
        if obj and obj.created_by is not None:
            return str(obj.created_by)
        elif obj and obj.created_by is None:
            return "Systemuser"
        return None
    _created_by.short_description = _("Created by")

    def _updated_by(self, obj=None):
        if obj and obj.updated_by is not None:
            return str(obj.updated_by)
        elif obj and obj.updated_by is None:
            return "Systemuser"
        return None
    _updated_by.short_description = _("Updated by")

    def is_active_badge(self, obj):
        return format_html(
            '<span class="badge bg-{} me-1"></span> {}', 'success' if obj.is_active else 'secondary', '',
        )
    is_active_badge.short_description = _("Active")

    def get_progressbar(self, obj):
        # Gibt eine Progressbar in HTML zurück.
        # Dafür muss das Model aber eine spalte "progress" besitzen
        # und mit einem Wert zwischen 0 und 100 befüllt sein.
        if not hasattr(self.model, 'progress'):
            return None
        value = 100 if obj.progress > 100 else obj.progress
        return format_html(
            f'''<div class="progress">
                <div class="progress-bar" style="width: {value}%" role="progressbar" aria-valuenow="{value}" aria-valuemin="0" aria-valuemax="100" aria-label="{value}% Complete">
                <span class="visually-hidden">{value}% Complete</span>
                </div>
            </div>'''
        )
    get_progressbar.short_description = _("Progress")

    def get_fieldsets(self, request, obj=None):
        # Hole die standardmäßigen Felder, die von Django automatisch generiert werden
        fieldsets = super().get_fieldsets(request, obj=obj)

        # Füge Adminfieldset hinzu
        if request.user.is_superuser:
            fieldsets += self.admin_fieldsets

        # Füge das benutzerdefinierte Feldset hinzu
        if self.show_sysdata:
            fieldsets += (
                (_("System data"), {
                    'classes': ('milea-system-data mt-3 col-lg-12',),
                    'fields': (('created_at', '_created_by', 'updated_at', '_updated_by'),),
                }),
            )

        return fieldsets

    def get_fields(self, request, obj=None):
        # Hole alle Felder, die von Django automatisch generiert werden
        fields = super().get_fields(request, obj=obj)

        # Entferne die Felder, die bereits im benutzerdefinierten Feldset enthalten sind
        fields = [field for field in fields if field not in ('is_active', 'created_at', '_created_by', 'updated_at', '_updated_by')]

        return fields

    # Allow search with verbose_id
    def get_search_results(self, request, queryset, search_term):
        # Hole den Verbose Id Tag aus dem Model und modifiziere den suchstring
        if self.model.OBJ_VERB_TAG and search_term.startswith('%s.' % self.model.OBJ_VERB_TAG):
            search_term = search_term.split('.')
            search_term = int(search_term[1])
        queryset, use_distinct = super().get_search_results(request, queryset, search_term)

        return queryset, use_distinct

    # Modify MileaChoices Queryset, this is maybe not the best place, but done is better than perfect
    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.__class__.__name__ == "MileaChoicesField":
            model = ContentType.objects.get_for_model(self.model)
            kwargs["queryset"] = MileaChoices.objects.filter(model=model)
        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        # Ridiculous override just to get rid of the "Hold down" text
        form_field = super().formfield_for_manytomany(db_field, request, **kwargs)
        msg = _("Hold down “Control”, or “Command” on a Mac, to select more than one.")
        form_field.help_text = form_field.help_text.replace(str(msg), '').strip()
        return form_field
